import json
import logging
from typing import Optional, List

import jsonpickle
import requests

import src
from src.api.entities.foto_sku import Foto_Sku
from src.api.entities.pedido import QueueOking
from src.api.entities.preco_lista import ListaPreco, ListaPrecoResponse
from src.database.queries import IntegrationType
from src.entities.invoice import Invoice
from src.entities.log import Log
from src.entities.response import InvoiceResponse, CatalogoResponse, OkvendasResponse
from src.jobs.system_jobs import OnlineLogger

# from src.database.entities.client_payment_plan import ClientPaymentPlan

send_log = OnlineLogger.send_log

logger = logging.getLogger()


def obj_dict(obj):
    return obj.__dict__


def object_list_to_dict(obj_list: list):
    lista = []
    for obj in obj_list:
        lista.append(obj.toJSON())
    return lista


def send_stocks(body):
    try:
        # auth = HTTPBasicAuth('teste@example.com', 'real_password')
        headers = {'Content-type': 'application/json',
                   'Accept': 'text/html',
                   'access-token': src.client_data.get('token_api_integracao')}
        url = src.client_data.get('url_api_principal') + '/catalogo/estoqueUnidadeDistribuicao'
        json_stock = jsonpickle.encode(body, unpicklable=False)
        if src.print_payloads:
            print(json_stock)
        response = requests.post(url, json=json.loads(json_stock), headers=headers)
        result = [OkvendasResponse(**t) for t in response.json()]
        if response.ok:
            return result
        else:
            if response.content is not None and response.content != '':
                return result

    except Exception as ex:
        logger.error(str(ex))


def post_prices(price: List):
    try:
        headers = {'Content-type': 'application/json',
                   'Accept': 'text/html',
                   'access-token': src.client_data.get('token_api_integracao')}
        url = src.client_data.get('url_api_principal') + '/catalogo/precos'
        json_prices = jsonpickle.encode(price, unpicklable=False)
        if src.print_payloads:
            print(json_prices)
        response = requests.post(url, json=json.loads(json_prices), headers=headers)
        result = [OkvendasResponse(**t) for t in response.json()]

        send_log('envia_preco_job', False, False, f'== Preço Enviado'
                 , 'info', 'PRECO', '')
        return result

    except Exception as ex:
        logger.error(f'Erro ao realizar POST na API OkVendas /catalogo/precos {str(ex)}')
        # PriceResponse([price.codigo_erp], 3, str(ex))


def post_invoices(invoice: Invoice) -> Optional[InvoiceResponse]:
    """
    Enviar NF de um pedido para api okvendas
    Args:
        invoice: Objeto com os dados da NF

    Returns:
    None se o envio for sucesso. Caso falhe, um objeto contendo status e descrição do erro
    """
    try:
        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': src.client_data.get('token_api_integracao')}
        url = src.client_data.get('url_api_principal') + '/pedido/faturar'
        json_invoice = jsonpickle.encode(invoice, unpicklable=False)
        if src.print_payloads:
            print(json_invoice)
        response = requests.post(url, json=json.loads(json_invoice), headers=headers)
        if response.ok:
            return None
        else:
            err = jsonpickle.decode(str(response.text))
            invoice_response = InvoiceResponse(**err)
            if '_okvendas' in invoice_response.message or '_openkuget' in invoice_response.message:
                invoice_response.message = 'Erro interno no servidor. Entre em contato com o suporte'
            return invoice_response

    except Exception as ex:
        return InvoiceResponse(IntegrationType.NOTA_FISCAL, str(ex))


def put_price_lists(body: List[ListaPreco]) -> List[ListaPrecoResponse]:
    try:
        url = f'{src.client_data.get("url_api_principal")}/catalogo/listapreco'
        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': src.client_data.get('token_api')}
        json_body = jsonpickle.encode(body, unpicklable=False)
        if src.print_payloads:
            print(json_body)
        response = requests.put(url, headers=headers, json=json.loads(json_body))
        if response.ok:
            res = jsonpickle.decode(response.content)
            return [ListaPrecoResponse(**r) for r in res]
        else:
            raise Exception(response.content)
    except Exception as e:
        logger.error(f'Erro ao realizar PUT /catalogo/listapreco na api okvendas {str(e)}')
        return []


def post_produtos(produtos: list):
    try:
        url = f'{src.client_data.get("url_api_principal")}/catalogo/produtos'

        json_produtos = jsonpickle.encode(produtos, unpicklable=False)
        if src.print_payloads:
            print(json_produtos)
        response = requests.post(url, json=json.loads(json_produtos), headers={
            'Content-type': 'application/json',
            'Accept': 'text/html',
            'access-token': src.client_data.get('token_api')})

        obj = jsonpickle.decode(response.content)
        result = []
        if 200 <= response.status_code <= 299:
            for res in obj:
                result.append(CatalogoResponse(**res))
        else:
            if type(obj) is list:
                for res in obj:
                    result.append(CatalogoResponse(**res))
            else:
                result.append(CatalogoResponse(**obj))

        return result
    except Exception as e:
        logger.error(f'Erro ao enviar produto para api okvendas {e}', exc_info=True)


def post_log(log: Log) -> bool:
    try:
        if not src.is_dev:
            url = f'{src.client_data.get("url_api_principal")}/log/logIntegracao'
            headers = {
                'Content-type': 'application/json',
                'Accept': 'application/json',
                'access-token': src.client_data.get('token_api')
            }

            json_log = jsonpickle.encode([log], unpicklable=False)
            if src.print_payloads:
                print(json_log)
            response = requests.post(url, json=json.loads(json_log), headers=headers)
            if response.ok:
                return True

            return False
        else:
            return True
    except Exception:
        return False


def put_photos_sku(body: List[Foto_Sku]):
    """
        Cadastra Fotos do SKU
        Podem ser enviadas até 50 fotos por vez.
        Args:
            body: Objeto com os dados das Fotos
        Returns:
        True se foi atualizado com Sucesso. False se não foi atualizado
    """
    url = f'{src.client_data.get("url_api_principal")}/v2/catalogo/fotos/sku'
    try:

        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': src.client_data.get('token_api')}

        json_photos = jsonpickle.encode(body, unpicklable=False)
        logger.info(f'{url} - {json_photos}')
        if src.print_payloads:
            print(json_photos)
        response = requests.put(url,
                                headers=headers,
                                json=json.loads(json_photos))
        if response.ok:
            return True
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
            return False
    except Exception as ex:
        logger.error(f'Erro ao realizar PUT /v2/catalogo/fotos/sku na api okvendas {url}' + str(ex), exc_info=True)
        return False


def get_order_queue_okvendas(status):
    queue = []
    url = ''
    try:
        url = src.client_data.get('url_api_principal') + f'/pedido/fila/{status}'
        headers = {'Content-type': 'application/json',
                   'Accept': 'application/json',
                   'access-token': src.client_data.get('token_api_integracao')}
        response = requests.get(url, headers=headers)
        if response.ok:
            queue = [QueueOking(**t) for t in response.json()['fila']]
        else:
            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
    except Exception as ex:
        logger.error(f'Erro ao realizar GET na api MPLACE {url}' + str(ex), exc_info=True)
        raise

    return queue
